"""This module contains the settings for the negotium application.
"""

# default settings
DEFAULT_HOST = 'localhost'
DEFAULT_PORT = 6379
DEFAULT_USERNAME = 'guest'
DEFAULT_PASSWORD = 'guest'
DEFAULT_QUEUE = 'negotium_queue'
DEFAULT_SCHEDULER_QUEUE = 'negotium_scheduler_queue'
DEFAULT_SCHEDULER_SORTED_SET = 'negotium_scheduler_sorted_set'
DEFAULT_LOGFILE = 'negotium.log'

# logging settings
LOGGING_FORMAT = '[%(asctime)s] [%(name)s] [%(levelname)s] %(message)s\n'
LOGGING_DATE_FORMAT = '%Y-%m-%d %H:%M:%S'



import datetime
import json
import redis

from negotium.settings import DEFAULT_QUEUE, DEFAULT_SCHEDULER_QUEUE, DEFAULT_SCHEDULER_SORTED_SET
from negotium.utils.logger import log

class _Publisher:
    def __init__(self, db: int, host: str, port: int, app_name: str, logfile: str=None):
        self.connection = None
        self.db = db
        self.host = host
        self.port = port
        self.app_name = app_name
        self.logfile = logfile

    def _create_connection(self):
        """Create a connection to the message broker
        """
        self.connection = redis.Redis(db=self.db, host=self.host, port=self.port)

    def _close_connection(self):
        """Close the connection
        """
        self.connection.close()

    def _publish(self, data: dict, eta: datetime.datetime=None):
        """Publish a message to the queue
        """
        log(self.logfile, data.get('app_name'), f"Received task: {data.get('function_name')}")
        if eta:
            data = {
                '_task': data,
                '_eta': eta.strftime('%Y-%m-%d %H:%M:%S.%f')
            }
            self.connection.rpush(DEFAULT_SCHEDULER_QUEUE + "__" + self.app_name, json.dumps(data))
            # zadd
            timestamp = datetime.datetime.strptime(eta.strftime('%Y-%m-%d %H:%M:%S.%f'), '%Y-%m-%d %H:%M:%S.%f').timestamp()
            self.connection.zadd(DEFAULT_SCHEDULER_SORTED_SET + "__" + self.app_name, {json.dumps(data): timestamp})
        else:
            self.connection.rpush(DEFAULT_QUEUE + "__" + self.app_name, json.dumps(data))




import datetime

from negotium.mq.publisher import _Publisher

def _delay(publisher: _Publisher, data: dict):
    """Execute the task
    """
    publisher._create_connection()
    publisher._publish(data)
    publisher._close_connection()

def _apply_async(publisher: _Publisher, data: dict, eta: datetime.datetime=None):
    """Schedule a task to be executed in the future
    """
    publisher._create_connection()
    publisher._publish(data, eta=eta)
    publisher._close_connection()
